#include <Arduino.h>
#include <TM1637Display.h>
#include <Bounce2.h>
#include <ctype.h>
#include <string.h>

int serial_putc(char c, FILE *)
{
    Serial.write(c);
    return c;
}


void printf_begin(void)
{
    fdevopen(&serial_putc, 0);
}

// Module connection pins (Digital Pins)
#define CLK A2
#define DIO A3

#define PIN_PAU_A A1
#define PIN_RUN_A A0

uint8_t keypins[]={PIN_PAU_A,PIN_RUN_A,8,9};
Bounce keys[]={Bounce(), Bounce()};
#define N_KEYS (sizeof(keys)/sizeof(keys[0]))
uint8_t keylock[N_KEYS];



TM1637Display display(CLK, DIO);

uint8_t ocet[]={
    '0',0x3F,'1',0x06,'2',0x5B,'3',0x4F,'4',0x66,'5',0x6D,'6',0x7D,'7',0x07,
'8',0x7F,'9',0x6F,'A',0x77,'a',0x5F,'b',0x7C,'C',0x39,'c',0x58,'d',0x5E,
'E',0x79,'e',0x7B,'F',0x71,'G',0x3D,'g',0x6F,'H',0x76,'h',0x74,'I',0x06,
'J',0x1E,'j',0x0E,'K',0x76,'L',0x38,'l',0x30,'M',0x55,'n',0x54,'O',0x3F,
'o',0x5C,'P',0x73,'q',0x67,'r',0x50,'S',0x6D,'t',0x78,'U',0x3E,'v',0x1C,
'W',0x2A,'X',0x76,'y',0x6E,'Z',0x5B,' ',0x00,'-',0x40,'*',0x63,'N',0x37,
'\0'
};

uint8_t mainMode=0, oldMainMode=0xff;
uint32_t lastCommand;
int mainPerCent, mainTimeLeft;
enum {
    MM_NC,
    MM_OFF,
    MM_ERROR,
    MM_PRINT,
    MM_PAUSE,
    MM_CANCEL,
    MM_HEAT,
    MM_IDLE
};
char *getToken(char **buf)
{
    char *c;
    while (**buf && isspace(**buf)) (*buf)++;
    if (!**buf) return NULL;
    c=*buf;
    while (**buf && !isspace(**buf)) (*buf)++;
    if (**buf) *(*buf)++=0;
    return c;
}

int getUToken(char **buf)
{
    char *c=getToken(buf);
    if (!c) return -1;
    if (!isdigit(*c)) return -1;
    return strtol(c,NULL,10);
}

static const struct {
    const char *cmd;
    uint8_t mm;
} mm_data[]={
    {"err",MM_ERROR},
    {"pau",MM_PAUSE},
    {"idl",MM_IDLE},
    {"ccl",MM_CANCEL},
    {"off",MM_OFF},
    {"prt",MM_PRINT},
    {"hea",MM_HEAT},
    {NULL,0}};
    
void doCmd(char *buf)
{
    char *cmd=getToken(&buf);if (!cmd) return;
    int i;
    for (i=0;mm_data[i].cmd;i++) if (!strcmp(cmd,mm_data[i].cmd)) break;
    if (!mm_data[i].cmd) return;
    if (mm_data[i].mm != MM_PRINT) {
        mainMode = mm_data[i].mm;
        return;
    }
    int p1=getUToken(&buf); if (p1<0 || p1 >99) return;
    int p2=getUToken(&buf); if (p2<0) return;
    mainMode = MM_PRINT;
    mainPerCent=p1;
    mainTimeLeft=p2;
}

 


void setup()
{
    Serial.begin(115200);
    int i;
    display.setBrightness(0x0f);
    printf_begin();
    for (i=0;i<N_KEYS; i++) {
        keys[i].attach(keypins[i], INPUT_PULLUP);
        keys[i].interval(50);
    }
}


uint8_t getSegments(uint8_t znak)
{
    int i;
    for (i=0;ocet[i];i+=2) if (znak == ocet[i]) return ocet[i+1];
    znak ^=32;
    for (i=0;ocet[i];i+=2) if (znak == ocet[i]) return ocet[i+1];
    return 0;
}

void drawText(char *txt)
{
    uint8_t data[4]={0,0,0,0};
    int i;
    for (i=0;i<4;i++) {
        
        if (!*txt) break;
        data[i] = getSegments(*txt++);
        if (*txt == ':') {
            if (i==1) data[i] |= 0x80;
            txt++;
        }
        
    }
    display.setSegments(data);
        
}


        
void getcmd(void)
{
    static char cmdbuf[32];
    static int cmdlen=0;
    while (Serial.available()) {
        int znak = Serial.read();
        if (znak == '\r' || znak == '\n') {
            if (!cmdlen) continue;
            cmdbuf[cmdlen]=0;
            doCmd(cmdbuf);
            cmdlen=0;
            lastCommand=millis();
            continue;
        }
        if (cmdlen < 31) cmdbuf[cmdlen++]=znak;
    }
    
}

static const struct {
    const char *str;
    uint8_t mm;
    uint8_t flags;
} mm_disp[]={
    {"PAUS",MM_PAUSE,1},
    {"IdLE",MM_IDLE,0},
    {"Cncl",MM_CANCEL,0},
    {"HEAT",MM_HEAT,0},
    {" nc ",MM_NC,0},
    {" -- ",MM_OFF,0},
    {NULL,0,0}};
    
void doDisplay()
{
    static uint32_t t= 0;
    static uint8_t pos=0;
    static char *errstr="Error Error";
    uint8_t force=0;
    if (mainMode != oldMainMode) {
        oldMainMode = mainMode;
        force=1;
        pos=0;
    }
    switch(mainMode) {
        case MM_ERROR:
        if (force || millis() - t > 150UL) {
            t= millis();
            pos = (pos+1) % 6;
            drawText(errstr+pos);
        }
        break;

        case MM_PRINT:
        if (force || millis() - t > 1000UL) {
            char buf[8];
            t= millis();
            if (!pos) sprintf(buf,"%02d P",mainPerCent);
            else if (mainTimeLeft < 6000) sprintf(buf,"%02d:%02d",mainTimeLeft / 60, mainTimeLeft % 60);
            else strcpy(buf," -:- ");
            pos = !pos;
            drawText(buf);
        }
        break;
        
        default:
        if (force || millis() - t > 1000UL) {
            int i;
            const char *cs;
            for (i=0;mm_disp[i].str;i++) if (mainMode == mm_disp[i].mm) break;
            cs=mm_disp[i].str; if (!cs) cs="-  -";
            t= millis();
            drawText((char *)((pos || !mm_disp[i].flags)? cs:"    "));
            pos=!pos;
        }        
    }
    
}
uint32_t startNCTimer;
static void askStatus()
{
    startNCTimer = millis();
    printf("#sta\r\n");
}

uint8_t getKey(uint8_t n)
{
    keys[n].update();
    if (keys[n].fell()) {
        keylock[n]=0;
        return 0;
    }
    if (keys[n].rose()) {
        uint8_t t=0;
        if (!keylock[n]) t=n +1;
        keylock[n]=0;
        return t;
    }
    if (!keys[n].read()) {
        if (keylock[n]) return 0;
        if (keys[n].duration() >=500) {
            keylock[n] = 1;
            return n + 9 ;
        }
    }
    
    return 0;
}
void loop()
{
    int i;
    for (i=0;i<N_KEYS;i++) {
        uint8_t n;
        if ((n=getKey(i))) printf("#key %d\r\n", n);
    }
    getcmd();
    if (mainMode != MM_NC && millis() -lastCommand > 30000UL) {
        mainMode = MM_NC;
        askStatus();
    }
    //if (mainMode == MM_NC && millis() - startNCTimer > 5000UL) {
    if (millis() - startNCTimer > 5000UL) {
        askStatus();
    }
        
    doDisplay();
}
